@@ -97,9 +97,10 @@ If STREAMING-P is non-nil, use the streaming endpoint."
    '(streaming embeddings model-list)
    (when-let ((model (llm-models-match (llm-gemini-chat-model provider)))
               (capabilities (llm-model-capabilities model)))
-     (append
-      (when (member 'tool-use capabilities) '(tool-use streaming-tool-use))
-      (seq-intersection capabilities '(image-input audio-input video-input))))))
+     (seq-uniq (append
+                (when (member 'tool-use capabilities) '(tool-use streaming-tool-use))
+                capabilities
+                (seq-intersection capabilities '(image-input audio-input video-input)))))))
 
 (cl-defmethod llm-models ((provider llm-gemini))
   (mapcar (lambda (model)
